/*
 * Copyright (C) 1999-2016. Christian Heller.
 *
 * This file is part of the Cybernetics Oriented Interpreter (CYBOI).
 *
 * CYBOI is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License,
 * or (at your option) any later version.
 *
 * CYBOI is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CYBOI. If not, see <http://www.gnu.org/licenses/>.
 *
 * Cybernetics Oriented Programming (CYBOP) <http://www.cybop.org/>
 * CYBOP Developers <cybop-developers@nongnu.org>
 *
 * @version CYBOP 0.18.0 2016-12-21
 * @author Christian Heller <christian.heller@tuxtax.de>
 */

#ifndef DATE_SOURCE
#define DATE_SOURCE

#include "../../constant/model/cyboi/log/level_log_cyboi_model.c"
#include "../../constant/model/cyboi/log/message_log_cyboi_model.c"
#include "../../constant/model/cyboi/state/integer_state_cyboi_model.c"
#include "../../constant/model/cyboi/state/pointer_state_cyboi_model.c"
#include "../../constant/name/cybol/logic/commander/date_commander_logic_cybol_name.c"
#include "../../constant/type/cyboi/state_cyboi_type.c"
#include "../../executor/accessor/getter/part/name_part_getter.c"
#include "../../executor/commander/date_commander.c"
#include "../../logger/logger.c"

#ifndef _MSC_VER
    #include <unistd.h>
#endif

/**
 * Get the date or change it
 *
 * Expected parametres:
 * - new date (optional): value of the new date
 *
 * - /T (optional): display option (display date, Win32 only)
 * - -I (optional): the ISO option (date in ISO format)
 * - -R (optional): the RFC option (date in RFC format)
 * - -u (optional): the UTC option (date in UTC format)
 *
 * Constraints:
 *
 * @param p0 the parametres data
 * @param p1 the parametres count
 * @param p2 the knowledge memory part
 * @param p3 the stack memory item
 */
void apply_date(void* p0, void* p1, void* p2, void* p3) {

    log_message_terminated((void*) INFORMATION_LEVEL_LOG_CYBOI_MODEL, (void*) L"Apply date.");

    // The new date part.
    void* n = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The display part.
    void* t = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The ISO part.
    void* i = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The RFC part.
    void* r = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The UTC part.
    void* u = *NULL_POINTER_STATE_CYBOI_MODEL;

    // The new date part model item.
    void* nm = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The display part model item.
    void* tm = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The ISO part model item.
    void* im = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The RFC part model item.
    void* rm = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The UTC part model item.
    void* um = *NULL_POINTER_STATE_CYBOI_MODEL;

    // The new date part model item data and count.
    void* nmd = *NULL_POINTER_STATE_CYBOI_MODEL;
    void* nmc = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The display part model item data.
    void* tmd = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The ISO part model item data.
    void* imd = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The RFC part model item data.
    void* rmd = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The UTC part model item data.
    void* umd = *NULL_POINTER_STATE_CYBOI_MODEL;

    // Get new date part.
    get_part_name((void*) &n, p0, (void*) DATE_DATE_COMMANDER_LOGIC_CYBOL_NAME, (void*) DATE_DATE_COMMANDER_LOGIC_CYBOL_NAME_COUNT, p1, p2, p3);
    // Get display part.
    get_part_name((void*) &t, p0, (void*) DISPLAY_DATE_COMMANDER_LOGIC_CYBOL_NAME, (void*) DISPLAY_DATE_COMMANDER_LOGIC_CYBOL_NAME_COUNT, p1, p2, p3);
    // Get ISO part.
    get_part_name((void*) &i, p0, (void*) ISO_DATE_COMMANDER_LOGIC_CYBOL_NAME, (void*) ISO_DATE_COMMANDER_LOGIC_CYBOL_NAME_COUNT, p1, p2, p3);
    // Get RFC part.
    get_part_name((void*) &r, p0, (void*) RFC_DATE_COMMANDER_LOGIC_CYBOL_NAME, (void*) RFC_DATE_COMMANDER_LOGIC_CYBOL_NAME_COUNT, p1, p2, p3);
    // Get UTC part.
    get_part_name((void*) &u, p0, (void*) UTC_DATE_COMMANDER_LOGIC_CYBOL_NAME, (void*) UTC_DATE_COMMANDER_LOGIC_CYBOL_NAME_COUNT, p1, p2, p3);

    // Get new date part model item.
    copy_array_forward((void*) &nm, n, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) MODEL_PART_STATE_CYBOI_NAME);
    // Get display part model item.
    copy_array_forward((void*) &tm, t, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) MODEL_PART_STATE_CYBOI_NAME);
    // Get ISO part model item.
    copy_array_forward((void*) &im, i, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) MODEL_PART_STATE_CYBOI_NAME);
    // Get RFC part model item.
    copy_array_forward((void*) &rm, r, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) MODEL_PART_STATE_CYBOI_NAME);
    // Get UTC part model item.
    copy_array_forward((void*) &um, u, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) MODEL_PART_STATE_CYBOI_NAME);

    // Get new date part model item data and count.
    copy_array_forward((void*) &nmd, nm, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) DATA_ITEM_STATE_CYBOI_NAME);
    copy_array_forward((void*) &nmc, nm, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) COUNT_ITEM_STATE_CYBOI_NAME);
    // Get display part model item data.
    copy_array_forward((void*) &tmd, tm, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) DATA_ITEM_STATE_CYBOI_NAME);
    // Get ISO part model item data.
    copy_array_forward((void*) &imd, im, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) DATA_ITEM_STATE_CYBOI_NAME);
    // Get RFC part model item data.
    copy_array_forward((void*) &rmd, rm, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) DATA_ITEM_STATE_CYBOI_NAME);
    // Get UTC part model item data.
    copy_array_forward((void*) &umd, um, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) DATA_ITEM_STATE_CYBOI_NAME);

    command_date(nmd, nmc, tmd, imd, rmd, umd);
}

/* DATE_SOURCE */
#endif
