/*
 * Copyright (C) 1999-2016. Christian Heller.
 *
 * This file is part of the Cybernetics Oriented Interpreter (CYBOI).
 *
 * CYBOI is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License,
 * or (at your option) any later version.
 *
 * CYBOI is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CYBOI. If not, see <http://www.gnu.org/licenses/>.
 *
 * Cybernetics Oriented Programming (CYBOP) <http://www.cybop.org/>
 * CYBOP Developers <cybop-developers@nongnu.org>
 *
 * @version CYBOP 0.18.0 2016-12-21
 * @author Christian Heller <christian.heller@tuxtax.de>
 */

#ifndef MEMORY_FREE_SOURCE
#define MEMORY_FREE_SOURCE

#include "../../constant/model/cyboi/log/level_log_cyboi_model.c"
#include "../../constant/model/cyboi/log/message_log_cyboi_model.c"
#include "../../constant/model/cyboi/state/integer_state_cyboi_model.c"
#include "../../constant/model/cyboi/state/pointer_state_cyboi_model.c"
#include "../../constant/name/cybol/logic/commander/memory_free_commander_logic_cybol_name.c"
#include "../../constant/type/cyboi/state_cyboi_type.c"
#include "../../executor/accessor/getter/part/name_part_getter.c"
#include "../../executor/commander/memory_free_commander.c"
#include "../../logger/logger.c"

#ifndef _MSC_VER
    #include <unistd.h>
#endif

/**
 * Shows the usage of the RAM.
 *
 * - human(optional): the human option (human-readable output)
 * - kilobytes(optional): the kilobytes option (print sizes in kilobytes)
 * - megabytes(optional): the megabytes option (print sizes in megabytes)
 * - gigabytes(optional): the gigabytes option (print sizes in gigabytes)
 * - total(optional): the total option (print total for RAM + swap)
 *
 * Constraints:
 *
 * @param p0 the parametres data
 * @param p1 the parametres count
 * @param p2 the knowledge memory part
 * @param p3 the stack memory item
 */

void apply_memory_free(void* p0, void* p1, void* p2, void* p3) {

    log_message_terminated((void*) INFORMATION_LEVEL_LOG_CYBOI_MODEL, (void*) L"Shows the usage of the RAM.");

    // The human part.
    void* h = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The kilobytes part.
    void* k = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The megabytes part.
    void* m = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The gigabytes part.
    void* g = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The total part.
    void* t = *NULL_POINTER_STATE_CYBOI_MODEL;

    // The human part.
    void* hm = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The kilobytes part.
    void* km = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The megabytes part.
    void* mm = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The gigabytes part.
    void* gm = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The total part.
    void* tm = *NULL_POINTER_STATE_CYBOI_MODEL;    

    // The human part.
    void* hmd = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The kilobytes part.
    void* kmd = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The megabytes part.
    void* mmd = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The gigabytes part.
    void* gmd = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The total part.
    void* tmd = *NULL_POINTER_STATE_CYBOI_MODEL;    
    
    // Get human part.
    get_part_name((void*) &h, p0, (void*) HUMAN_MEMORY_FREE_COMMANDER_LOGIC_CYBOL_NAME, (void*) HUMAN_MEMORY_FREE_COMMANDER_LOGIC_CYBOL_NAME_COUNT, p1, p2, p3);
    // Get kilobytes part.
    get_part_name((void*) &k, p0, (void*) KILOBYTES_MEMORY_FREE_COMMANDER_LOGIC_CYBOL_NAME, (void*) KILOBYTES_MEMORY_FREE_COMMANDER_LOGIC_CYBOL_NAME_COUNT, p1, p2, p3);
    // Get megabytes part.
    get_part_name((void*) &m, p0, (void*) MEGABYTES_MEMORY_FREE_COMMANDER_LOGIC_CYBOL_NAME, (void*) MEGABYTES_MEMORY_FREE_COMMANDER_LOGIC_CYBOL_NAME_COUNT, p1, p2, p3);
    // Get gigabytes part.
    get_part_name((void*) &g, p0, (void*) GIGABYTES_MEMORY_FREE_COMMANDER_LOGIC_CYBOL_NAME, (void*) GIGABYTES_MEMORY_FREE_COMMANDER_LOGIC_CYBOL_NAME_COUNT, p1, p2, p3);
    // Get total part.
    get_part_name((void*) &t, p0, (void*) TOTAL_MEMORY_FREE_COMMANDER_LOGIC_CYBOL_NAME, (void*) TOTAL_MEMORY_FREE_COMMANDER_LOGIC_CYBOL_NAME_COUNT, p1, p2, p3);
    
    // Get human part model item.
    copy_array_forward((void*) &hm, h, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) MODEL_PART_STATE_CYBOI_NAME);
    // Get kilobytes part model item.
    copy_array_forward((void*) &km, k, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) MODEL_PART_STATE_CYBOI_NAME);
    // Get megabytes part model item.
    copy_array_forward((void*) &mm, m, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) MODEL_PART_STATE_CYBOI_NAME);
    // Get gigabytes part model item.
    copy_array_forward((void*) &gm, g, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) MODEL_PART_STATE_CYBOI_NAME);
    // Get total part model item.
    copy_array_forward((void*) &tm, t, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) MODEL_PART_STATE_CYBOI_NAME);

    // Get human part model item.
    copy_array_forward((void*) &hmd, hm, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) DATA_ITEM_STATE_CYBOI_NAME);
    // Get kilobytes part model item.MOD
    copy_array_forward((void*) &kmd, km, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) DATA_ITEM_STATE_CYBOI_NAME);
    // Get megabytes part model item.
    copy_array_forward((void*) &mmd, mm, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) DATA_ITEM_STATE_CYBOI_NAME);
    // Get gigabytes part model item.
    copy_array_forward((void*) &gmd, gm, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) DATA_ITEM_STATE_CYBOI_NAME);
    // Get total part model item.
    copy_array_forward((void*) &tmd, tm, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) DATA_ITEM_STATE_CYBOI_NAME);

    command_memory_free(hmd, kmd, mmd, gmd, tmd);
}

/* MEMORY_FREE_SOURCE */
#endif
