/*
 * Copyright (C) 1999-2016. Christian Heller.
 *
 * This file is part of the Cybernetics Oriented Interpreter (CYBOI).
 *
 * CYBOI is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License,
 * or (at your option) any later version.
 *
 * CYBOI is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CYBOI. If not, see <http://www.gnu.org/licenses/>.
 *
 * Cybernetics Oriented Programming (CYBOP) <http://www.cybop.org/>
 * CYBOP Developers <cybop-developers@nongnu.org>
 *
 * @version CYBOP 0.18.0 2016-12-21
 * @author Christian Heller <christian.heller@tuxtax.de>
 */

#ifndef USERLOG_SOURCE
#define USERLOG_SOURCE

#include "../../constant/model/cyboi/log/level_log_cyboi_model.c"
#include "../../constant/model/cyboi/log/message_log_cyboi_model.c"
#include "../../constant/model/cyboi/state/integer_state_cyboi_model.c"
#include "../../constant/model/cyboi/state/pointer_state_cyboi_model.c"
#include "../../constant/name/cybol/logic/commander/userlog_commander_logic_cybol_name.c"
#include "../../constant/type/cyboi/state_cyboi_type.c"
#include "../../executor/accessor/getter/part/name_part_getter.c"
#include "../../executor/commander/userlog_commander.c"
#include "../../logger/logger.c"

#ifndef _MSC_VER
    #include <unistd.h>
#endif

/**
 * shows the Userlog
 *
 * Expected parametres:
 * - none
 * 
 * Options: -h , u, s, o
 *
 * Constraints:
 * @param p0 the parametres data
 * @param p1 the parametres count
 * @param p2 the knowledge memory part
 */
void apply_userlog(void* p0, void* p1, void* p2, void* p3) {

       log_message_terminated((void*) INFORMATION_LEVEL_LOG_CYBOI_MODEL, (void*) L"Apply userlog.");

    // The noheader part.
    void* h = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The no current part.
    void* u = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The short part.
    void* s = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The old part.
    void* o = *NULL_POINTER_STATE_CYBOI_MODEL;


    // The noheader part model item.
    void* hm = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The nocurrent part model item.
    void* um = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The short part model item.
    void* sm = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The old part model item.
    void* om = *NULL_POINTER_STATE_CYBOI_MODEL;



    // The noheader part model item data and count.
    void* hmd = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The nocurrent part model item data and count.
    void* umd = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The short part model item data and count.
    void* smd = *NULL_POINTER_STATE_CYBOI_MODEL;
    // The old part model item data and count.
    void* omd = *NULL_POINTER_STATE_CYBOI_MODEL;



    // Get noheader part.
    get_part_name((void*) &h, p0, (void*) NOHEADER_USERLOG_COMMANDER_LOGIC_CYBOL_NAME, (void*) NOHEADER_USERLOG_COMMANDER_LOGIC_CYBOL_NAME_COUNT, p1, p2, p3);
    // Get nocurrent part.
    get_part_name((void*) &u, p0, (void*) NOCURRENT_USERLOG_COMMANDER_LOGIC_CYBOL_NAME, (void*) NOCURRENT_USERLOG_COMMANDER_LOGIC_CYBOL_NAME_COUNT, p1, p2, p3);
    // Get short part.
    get_part_name((void*) &s, p0, (void*) SHORT_USERLOG_COMMANDER_LOGIC_CYBOL_NAME, (void*) SHORT_USERLOG_COMMANDER_LOGIC_CYBOL_NAME_COUNT, p1, p2, p3);
    // Get old part.
    get_part_name((void*) &o, p0, (void*) OLD_USERLOG_COMMANDER_LOGIC_CYBOL_NAME, (void*) OLD_USERLOG_COMMANDER_LOGIC_CYBOL_NAME_COUNT, p1, p2, p3);


    // Get noheader part model item.
    copy_array_forward((void*) &hm, h, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) MODEL_PART_STATE_CYBOI_NAME);
    // Get nocurrent part model item.
    copy_array_forward((void*) &um, u, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) MODEL_PART_STATE_CYBOI_NAME);
    // Get short part model item.
    copy_array_forward((void*) &sm, s, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) MODEL_PART_STATE_CYBOI_NAME);
    // Get old part model item.
    copy_array_forward((void*) &om, o, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) MODEL_PART_STATE_CYBOI_NAME);

    // Get noheader part model item data and count.
    copy_array_forward((void*) &hmd, hm, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) DATA_ITEM_STATE_CYBOI_NAME);
    // Get nocurrent part model item data and count.
    copy_array_forward((void*) &umd, um, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) DATA_ITEM_STATE_CYBOI_NAME);
    // Get short part model item data and count.
    copy_array_forward((void*) &smd, sm, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) DATA_ITEM_STATE_CYBOI_NAME);
    // Get old part model item data and count.
    copy_array_forward((void*) &omd, om, (void*) POINTER_STATE_CYBOI_TYPE, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, (void*) DATA_ITEM_STATE_CYBOI_NAME);

    command_userlog(hmd, umd, smd, omd);
}


/* USERLOG_SOURCE */
#endif
